/*************************************************************************** 
 *   Copyright (C) 2007 by apterium   *                                      
 *   apterium@gmail.com   *                                                  
 *                                                                         * 
 *   This program is free software; you can redistribute it and/or modify  * 
 *   it under the terms of the GNU General Public License as published by  * 
 *   the Free Software Foundation; either version 3 of the License, or     * 
 *   (at your option) any later version.                                   * 
 *                                                                         * 
 *   This program is distributed in the hope that it will be useful,       * 
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 *   GNU General Public License for more details.                          * 
 *                                                                         * 
 *   You should have received a copy of the GNU General Public License     * 
 *   along with this program; if not, write to the                         * 
 *   Free Software Foundation, Inc.,                                       * 
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 ***************************************************************************/

#ifndef __TEXT_H
#define __TEXT_H

#include <vector>
#include <QString>
#include <gmpxx.h>

/**
 * @short Main class for manipulating plaintexts and ciphertexts for classical
 * ciphers.
 * This class is designed to ease the conversion between QStrings as used by the
 * user interface and numbers as used by libcrypto and libalgebra. The main goal
 * of this class is to be able to convert a text given by the user, possibly
 * with characters which are not in the considered alphabet for encryption and
 * decryption. For example, a plaintext may have spaces that the user keep in the
 * ciphertext.
 * 
 * @todo Except some UI-related things, as printing a matrix with letters and
 * with numbers, it is in this class and NOWHERE else that the a=0 or a=1 option
 * shall be taken into account. So, a short list of things to do : 1 Add a
 * parameter a=0 or a=1 to the class. 2 Use it in the conversion. For the ease
 * of the underlying libraries, which use numbers between 0 and N-1 modulo N (as
 * do every mathematician ;-), the option should be carefully handled in the
 * conversions. 
 * @author apterium 
 */
class Text
{
public:
	/**
	 * Constructs a new Text instance given it QString representation and
	 * the alphabet.
	 * 
	 * @param strng a constant reference to the string value of this text.
	 * @param alphbt a pointer to the alphabet, which this Text will
	 * memorize.
	 * @param cs the case sensitivity option. 
	 */
	Text(const QString& strng, const QString* alphbt, const Qt::CaseSensitivity cs = Qt::CaseSensitive);

	~Text();
	
	/**
	 * Gets the integer representation of this text. Recommended : use for
	 * encryption or decryption.
	 * 
	 * @return a constant reference to the vector of integers representing
	 * the sequence of characters of the text which appear in the alphabet.
	 */
	const std::vector<mpz_class>& getIntRepr() const;
	
	/**
	 * Gets the number of characters of the text which appears in the
	 * alphabet. That is, the length of the integer representation.
	 * Recommended use is to be able to know how to do the padding, for
	 * example in Hill cipher the lenght must be a multiple of the key
	 * dimension.
	 * 
	 * @return the length of the integer representation of the text.
	 */
	std::vector<mpz_class>::size_type getWorkTextLength() const;

	/**
	 * Add the first character of the alphabet at the end of the
	 * string. That is used for padding.
	 * 
	 */
	void addZero();

	/**
	 * Sets the resulting text. That is, for example, the result of the
	 * cipher on the integer representation of the plaintext.
	 * 
	 * Note that it MUST use only integers which are in the range of the
	 * size of the alphabet, of course...
	 */
	void setResultingText(const std::vector<mpz_class>& resText);

	/**
	 * Gets the resulting QString after having set the resulting text.
	 * 
	 * @return a constant reference to the resulting text.
	 */
	const QString& getResultingText()
	{
		return *(this->resultString);
	}

	/**
	 * Gets the length of the alphabet, that is, the modulus of the
	 * crypto computations. For example, with the normal alphabet, it is 26.
	 * 
	 */
	inline mpz_class getAlphabetCardinality() const;
	
private:
	/**
	 * The value of the initial QString. Is memorized in order to ease the
	 * reconstruction of a text : given an integer vector containing the
	 * index of the characters in the alphabet, somewhere there will be
	 * non-alphabet characters, at some positions. The QString will allow us
	 * to find them and replace them back at the right place.
	 * Note that this string is the original text, for example the plaintext
	 * in a classical cipher.
	 * 
	 */
	QString* initialString;

	/**
	 * The resulting text of the computation. For example, the ciphertext.
	 * 
	 * 
	 */
	QString* resultString;

       	/**
	 * A constant pointer to the alphabet.
	 * 
	 */
	const QString* alphabet;

	Qt::CaseSensitivity caseSens;
	
	/**
	 * The mpz_class representation of the initial text. All elements shall
	 * take value between 0 and the size of the alphabet.
	 * 
	 */
	std::vector<mpz_class>* initIntRep;
	
	/**
	 * The computation result, that is, the encryption/decryption of
	 * initIntRep.
	 *
	 */
	std::vector<mpz_class>* resultIntRep;

	/**
	 * Computes the integer representation from the QString and the
	 * alphabet.
	 * Note that it does not store the characters which don't appear in the
	 * alphabet in the integer representation. 
	 * 
	 */
	void updateIntRep();

	/**
	 * Computes the resulting string, after having set the resulting integer
	 * representation of the text.
	 * 
	 */
	void updateStringRep();
};


#endif //__TEXT_H
