/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_CLASSICAL_ANALYSIS_FREQUENCIES_H
#define __LIB_CLASSICAL_ANALYSIS_FREQUENCIES_H

#include <vector>
#include <gmpxx.h>


namespace libClassicalAnalysis
{
	/**
	 * @short Frequencies of letters computation on a text.
	 * This class computes the frequencies of letters, digram, trigram,
	 * (n-gram ?) in a given text. This is the basis of classical cipher
	 * cryptanalysis.
	 *
	 * @author apterium
	 */
	class Frequencies
	{
	public:
		/**
		 * Constructs a new frequencies analysis on a given text and a
		 * given alphabet. Note that the alphabet is mandatory because
		 * we should analyse texts which do not contain some letters.
		 * 
		 * @param t a vector of mpz_class containing the (cipher)text
		 * representation, for example as given by the Text class of
		 * cryptoolinux.
		 * @param alph a vector of mpz_class. alpha.at(i) is the number
		 * representing the ith character of the user-defined
		 * alphabet. For example, it will be [0...25] for the
		 * "normal" alphabet.
		 */
		Frequencies(const std::vector<mpz_class>& t, const std::vector<mpz_class>& alph);
		
		/**
		 * Destructor.
		 * 
		 */
		~Frequencies();

		/**
		 * Computes the frequencies of the unique letters.
		 * 
		 * @return a vector of mpz_class r, r(i) = the number of times
		 * the ith letter of the alphabet appear in the text.
		 */
		std::vector<mpz_class> getSingleFrequencies();
		
		/**
		 * Gets the frequences of sequence of two consecutive letters.
		 * 
		 * @return a vector of vector of mpz_class r. r.at(i).at(j) =
		 * frequence of apparition of alphabet.at(i) alphabet.at(j).
		 */
		std::vector<std::vector<mpz_class> > getTwoFrequencies();
		
	private:
		/**
		 * The text.
		 * 
		 */
		std::vector<mpz_class> *text;
		
		/**
		 * The alphabet.
		 * 
		 */
		std::vector<mpz_class> *alphabet;
	};

}

#endif
