/*************************************************************************** 
 *   Copyright (C) 2007 by apterium   *                                      
 *   apterium@gmail.com   *                                                  
 *                                                                         * 
 *   This program is free software; you can redistribute it and/or modify  * 
 *   it under the terms of the GNU General Public License as published by  * 
 *   the Free Software Foundation; either version 3 of the License, or     * 
 *   (at your option) any later version.                                   * 
 *                                                                         * 
 *   This program is distributed in the hope that it will be useful,       * 
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        * 
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         * 
 *   GNU General Public License for more details.                          * 
 *                                                                         * 
 *   You should have received a copy of the GNU General Public License     * 
 *   along with this program; if not, write to the                         * 
 *   Free Software Foundation, Inc.,                                       * 
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             * 
 ***************************************************************************/

#ifndef __LIB_CLASSICAL_CRYPTO_HILL_ENGINE_H
#define __LIB_CLASSICAL_CRYPTO_HILL_ENGINE_H

#include <iostream>
#include <vector>
#include <gmpxx.h>
#include "../libalgebra/libalgebra.h"
#include "InvalidKeyException.h"
#include "CipherEngine.h"

namespace LibClassicalCrypto
{
	/**
	 * Hill cipher engine. 
	 * 
	 * @author apterium
	 */
	class HillEngine : public CipherEngine
	{
	public:
		/**
		 * Constructs a new HillEngine instance, given the size of the
		 * alphabet and a key.
		 * 
		 * @param matrix the key. Won't be deleted in any way by this class.
		 * @param modulus the size of the alphabet.
		 * @param dim the dimension (number of row = number
		 * columns) of the matrix.
		 * @throw InvalidKeyException if the matrix is not invertible.
		 */
		HillEngine(int** matrix,  const mpz_class& modulus, int dim)
			throw(InvalidKeyException);
		
		/**
		 * Default constructor. The engine MUST be intialised with a try
		 * { } catch { } block immediately after declaration, using the
		 * previous constructor.
		 *
		 */
		HillEngine();
		
		/**
		 * Copy constructor.
		 * 
		 */
		HillEngine(const HillEngine& copy);

		/**
		 * Destructor.
		 * 
		 */
		virtual ~HillEngine();
		
		/**
		 * @see CipherEngine
		 * Note that the plaintext MUST be padded, that is, its length
		 * must be a multiple of the dimension of the matrix key.
		 * 
		 */
		std::vector<mpz_class> encipher(const std::vector<mpz_class>& plainText) const;
		
		/**
		 * Stores the content of the inverse of the key into a int**
		 * matrix. Designed to be used in cryptanalysis (?) or in the
		 * explanation method of the GUI.
		 * 
		 * @param matrix the matrix in which the inverse of the key is
		 * to be stored. It is assumed to have enough memory allocated !
		 * @param length the dimension of the previous matrix, with
		 * respect to the allocated memory.
		 * @throw DimensionMismatchException iff length and the recorded
		 * dimensions do not match.
		 * @todo beware of type conversion (unsigned long to
		 * int). Better would be to only be more careful with the use of
		 * thoses functions...
		 */
		void getInverseKey(int** matrix, int length)
			throw(libalgebra::DimensionMismatchException);
		
		/**
		 * @see CipherEngine.
		 * Note that, as for encipher, the length of the ciphertext MUST
		 * be a multiple of the dimension of the key.
		 */
		std::vector<mpz_class> decipher(const std::vector<mpz_class>& cipherText) const;
		
		/**
		 * Debugging help. The engine is printed to the output stream.
		 * 
		 */
		friend std::ostream& operator<<(std::ostream& out, const HillEngine& engine);
		
	private:
		/**
		 * The key, under the proper form.
		 * 
		 */
		libalgebra::DenseMatrix<libalgebra::RingZnElement,
					libalgebra::RingZn>* key;

		
		/**
		 * The inverse of the key. That is, the key for decryption.
		 *
		 */
		libalgebra::DenseMatrix<libalgebra::RingZnElement,
					libalgebra::RingZn>* invKey;

		/**
		 * The dimension of the key.
		 * 
		 */
		int dimension;
		
		/**
		 * The RingZn used to do every computations.
		 * 
		 */
		libalgebra::RingZn* ring;

		/**
		 * Real cipher function. It operates only on RingZnElements.
		 * 
		 * @param text the text. It will be cut into blocks of the
		 * dimension of the key and multiplied by the matrix.
		 * @param matrix the key for encryption, its inverse for decryption.
		 */
		std::vector<libalgebra::RingZnElement> compute(const std::vector<libalgebra::RingZnElement>& plainText,
							       const libalgebra::DenseMatrix<libalgebra::RingZnElement, libalgebra::RingZn>& matrix) const;
		
		/** 
		 * Proceed with the multiplication of one block with the given
		 * matrix. The dimension MUST be appropriate !
		 * 
		 */
		std::vector<libalgebra::RingZnElement> proceedBlock(const std::vector<libalgebra::RingZnElement>& textBlock,
		    const libalgebra::DenseMatrix<libalgebra::RingZnElement, libalgebra::RingZn>& matrix) const;
	};
	
} //namespace LibClassicalCrypto

#endif //__LIB_CLASSICAL_CRYPTO_HILL_ENGINE_H
