/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_EXTENSION_FIELD_H
#define __LIB_ALGEBRA_EXTENSION_FIELD_H

#include <gmpxx.h>
#include "Field.h"
#include "DensePolynomial.h"
#include "RingZnElement.h"

namespace libalgebra
{
	
	/**
	 * @short Finite extension Fields. That is, if K is an underlying field,
	 * ExtensionField is K[X]/P where P is an irreductible polynomial on K.
	 *
	 * It may notably be used for Galois Field, and will be used for integer
	 * rings. Note that it may also be used for extension Rings, but be
	 * careful with mathematical requirements... T is the base RingElement
	 * type, R is the base Ring type. T and R must obviously correspond.
	 * 
	 * @todo replace the use of a DensePolynom by the use of a sparse polynom, and
	 * add the correct operators in DensePolynom. It will be better as very sparse
	 * polynom are efficient for the implementation of finite fields.
	 * @author apterium
	 */
	template<class T, class R>
		class ExtensionField : public Ring
	{
	public:
		/**
		 * Creates a new ExtensionField instance. The parameter shall be prime and will
		 * be refused if it's not the case with high probability.
		 *
		 * @todo this constructor should instead accept any generic polynomial.
		 * @param polynomial a constant reference to the base polynom
		 * for representing this Extension field.
		 */
		ExtensionField(const DensePolynomial<T, R>& polynomial);
		// GENERIC EXTENSION FIELD : it's the same for
		//		Fp[X]/(X^n+kX+k') and Q[X]/[X^N+34] for example
		//		! Due to the genericity of templates, the
		//		solution should be pretty...
		// and the problems which may arise
		// if the polynomial is not irreductible will be
		// immediately correctly handled : exceptions will go through
		// the call until reach the user who instanciated this.
		
		~ExtensionField();
		
		/**
		 * @see Field. 
		 * Will have to be specialised among different instances of the
		 * underlying Ring or Field.
		 */
		mpz_class characteristic() const;
		
		/**
		 * @see Field
		 */
		mpz_class cardinality() const;
		
		/**
		 * 
		 * @see Ring
		 * @todo properly handle this function. For example, this Ring
		 * is an integral domain if the polynomial is irreductible (hard
		 * to determine) and the base ring is a field.
		 */
		inline bool isIntegralDomain() const
		{
			return false;
		};
		
		/**
		 * This ring is not Euclidean.
		 *
		 * @see Ring. 
		 */
		inline bool isEuclidean() const
		{
			return false;
		};
		
		/**
		 * Gets the polynomial representation of 0 in this extension
		 * field. That is, the (supposed) irreductible polynomial P such
		 * that this field = F[X]/P(X).
		 * 
		 * @return a constant reference to the base polynomial of this
		 * extension field.
		 * 
		 */
		inline const DensePolynomial<T, R>& getPolynomial() const
		{
			return *(this->polynomial);
		};
		
		
	private:
		DensePolynomial<T, R>* polynomial;
	};
	
	
} //namespace libalgebra

#include "ExtensionField.tpp"

#endif //__LIB_ALGEBRA_EXTENSION_FIELD_H
