/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef __LIB_ALGEBRA_EXTENSION_FIELD_ELEMENT_H
#define __LIB_ALGEBRA_EXTENSION_FIELD_ELEMENT_H

#include <iostream>
#include "ExtensionField.h"
#include "NonIrreductiblePolynomialException.h"
#include "NonInvertibleElementException.h"

namespace libalgebra
{
	/**
	 * Implementation of Extension Field elements, using a polynomial
	 * representation. 
	 * T is the base ring element type, R is the base ring type.
	 * 
	 * @author apterium
	 */
	template<class T, class R>
		class ExtensionFieldElement : public RingElement
	{
	public:
		/**
		 * Constructs a new ExtensionFieldElement, given a polynomial
		 * which represents it modulo the base polynomial of the field.
		 * 
		 * @param poly a constant reference to the DensePolynomial which
		 * represents this.
		 * @param the field which this element belongs to. As is
		 * RingZnElement.
		 */
		ExtensionFieldElement(const DensePolynomial<T, R>& poly, 
				      const ExtensionField<T, R>* f);
		
		/**
		 * Copy constructor.
		 *
		 * @param element a constant reference to the element which
		 * shall be copied into a new one.
		 */
		ExtensionFieldElement(const ExtensionFieldElement<T, R>& element);
		
		/**
		 * Destructor.
		 *
		 */
		~ExtensionFieldElement();
		
		/**
		 * Addition of an ExtensionFieldElement to this one.
		 *
		 * @param right a constant reference to the right operand of the
		 * addition.
		 * @return *this + right
		 */
		ExtensionFieldElement<T, R> operator+(const ExtensionFieldElement<T, R>& right) const;
		
		/**
		 * Subtraction of an ExtensionFieldElement to this one.
		 * 
		 * @param right a constant reference to the
		 * ExtensionFieldElement which shall be subtracted to this one.
		 * @return *this - right.
		 */
		ExtensionFieldElement<T, R> operator-(const ExtensionFieldElement<T, R>& right) const;
		
		/**
		 * Multiplication of an ExtensionFieldElement by this one.
		 *  
		 * @param right the second multiplication operand.
		 * @return (*this) * right
		 */
		ExtensionFieldElement<T, R> operator*(const ExtensionFieldElement<T, R>& right) const;
		
		/**
		 * Division of this ExtensionFieldElement by an other one. this
		 * is the dividend of the operation.
		 * 
		 * @param divisor a constant reference to the divisor. 
		 * @return (*this) / divisor
		 * @throw NonInvertibleElementException<T> if divisor is not invertible.
		 */
		ExtensionFieldElement<T, R> operator/(const ExtensionFieldElement<T, R>& divisor) 
			const throw(NonInvertibleElementException<T>);
		
		/**
		 * Computes the inverse of this ExtensionFieldElement.
		 * 
		 * @todo replace the function of GenericAlgorithms which is used
		 * by a more appropriate one, which should do modular inversion.
		 * @return the inverse of *this.
		 * @throw NonInvertibleElementException<T> if this element is
		 * not invertible.
		 */
		ExtensionFieldElement<T, R> inverse() const 
			throw(NonIrreductiblePolynomialException, 
			      NonInvertibleElementException<ExtensionFieldElement<T, R> > );

		/**
		 * @see RingElement
		 * 
		 */
		bool isInvertible() const;

		inline const ExtensionField<T, R>& getRing() const
		{
			return ;
		};
		
	private:
		//the value
		DensePolynomial<T, R>* polynomial; 

		const ExtensionField<T, R>* field;
		/**
		 * Pretty printing of an Extension Field Element. Basically, prints its
		 * polynomial representation to out.
		 *
		 * @param out a reference to the outputstream where this object
		 * shall be sent.
		 * @param element a constant reference to the element.
		 * @return a reference to the resulting outputstream.
		 */
		template<class U, class RR>
		friend std::ostream& operator<<(std::ostream& out, 
						const ExtensionFieldElement<U, RR> element);
	
	};
	
	
	
	
} //namespace libalgebra

#include "ExtensionFieldElement.tpp"

#endif //__LIB_ALGEBRA_EXTENSION_FIELD_ELEMENT_H
