/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __LIB_ALGEBRA_RING_Z_N_H
#define __LIB_ALGEBRA_RING_Z_N_H

#include <gmpxx.h>
#include <map>
#include "Ring.h"


namespace libalgebra
{
	/**
	 * Implementation of the Z/nZ ring, when n is an integer.
	 *
	 * @author apterium
	 */
	class RingZn : public Ring
	{
	public:
		/**
		 * Constructs a new RingZn instance.
		 * 
		 * @param n the modulus of the ring.
		 */
		RingZn(const mpz_class &n);
		
		/**
		 * Copy constructor.
		 * 
		 * @param copy a constant reference to the RingZn to be copied.
		 */
		RingZn(const RingZn& copy);
		/**
		 * Constructs a new RingZn instance, from the list of coprime numbers which
		 * composes n. They may for example be the list of prime numbers dividing n,
		 * with their maximal exponent in n.
		 *
		 * @todo replace the following with a hash_map (unordered_map). When it will
		 * be standardised of course...
		 * @param factors a reference towards the list of coprime factors of n,
		 * whose product is n. Key = the factor, Value = its maximal exponent in n.
		 * @todo really use it (with the Chineses Remainder
		 * Theorem). Currently it's only dirty converted to the resulting int.
		 * @remarks will throw an exception if the factors are not
		 * pairwise coprime. So do not use yet, or expect breakage.
		 */
		RingZn(const std::map<mpz_class, mpz_class> &factors);
		
		~RingZn();

		/**
		 * @see Field
		 */
		mpz_class characteristic() const
		{
			mpz_class &ret = (*this->modulus);
			return ret;
		};
		
		/**
		 * @see Field
		 */ 
		mpz_class cardinality() const
		{
			mpz_class &ret = *(this->modulus);
			return ret;
		};


		/**
		 * Gets the base modulus p of the field.
		 *
		 * @return a constant reference to the modulus p of this Z/pZ
		 * implementation. It shall not be modified...
		 */    
		inline const mpz_class& getModulus() const
		{
			return *(this->modulus);
		};
		
		/**
		 * Uses the underlying Gnu Multiple Precision Library to determine
		 * whether the modulus is prime or not. That is, some trial
		 * divisions, then a fixed number of  Miller Rabin tests.
		 * 
		 * @todo better handle the result of GMP (2 = prime, 1 = probably
		 * prime, 0 = composite)
		 * @todo do NOT hardcode the number of rounds. See it with a
		 * configuration file...
		 * @todo create a number theoretic library and let the user decide
		 * which test to run (if he does cryptography then compositeness
		 * tests are sufficient, if he wants to do pure mathematics then
		 * deterministic tests are mandatory).
		 * @see Ring
		 */
		bool isIntegralDomain() const;

		/**
		 * @see Ring
		 * 
		 */
		inline bool isEuclidean() const
		{
			return false;
		};
		
	private:
		mpz_class *modulus;
		
		/**
		 * Converts the list of coprime factors of n, with their maximal
		 * exponent in n, to n.
		 * 
		 */
		mpz_class* convert(const std::map<mpz_class, mpz_class> &factors) const;
		
	};

}  
  
  



#endif // __LIB_ALGEBRA_RING_Z_N_H
