/***************************************************************************
 *   Copyright (C) 2007 by apterium   *
 *   apterium@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <gmp.h>
#include "RingZnElement.h"

namespace libalgebra
{
 	RingZnElement::RingZnElement(const RingZn* r)
 	{
 		this->value = new mpz_class(0);
		this->ring = r;
 	}
	
	RingZnElement::RingZnElement(const mpz_class &k, const RingZn* r)
	{
		this->value = new mpz_class(k.get_mpz_t());
		this->ring = r;
		*value %= r->getModulus();
	}
	
	RingZnElement::RingZnElement(const RingZnElement &copy) 
		: RingElement::RingElement(copy)
	{
		this->ring = copy.ring;
		this->value = new mpz_class(copy.value->get_mpz_t());
	}
	
	RingZnElement::~RingZnElement()
	{
		if (this->value != NULL) delete this->value;
	}
	
	RingZnElement RingZnElement::operator+(const RingZnElement& right) const
		throw(RingMismatchException)
	{
		if (this->ring != right.ring) {
			throw(RingMismatchException());
		}
		mpz_class n = this->ring->getModulus();
		mpz_class result = (this->getValue() + right.getValue()) % n;
		RingZnElement res  = RingZnElement(result, this->ring);
		return res;
	}

	void RingZnElement::operator+=(const RingZnElement& right)
		  throw(RingMismatchException)
	{
		if (this->ring != right.ring) {
			throw(RingMismatchException());
		}
		*(this->value) = *(this->value) + *(right.value);
		*(this->value) %= ring->getModulus();
	}
	
	RingZnElement RingZnElement::operator-(const RingZnElement& right) const
		throw(RingMismatchException)
	{ 
		if (this->ring != right.ring) {
			throw(RingMismatchException());
		}
		mpz_class n = this->ring->getModulus();
		mpz_class result = (this->getValue() - right.getValue()) % n;
		RingZnElement res  = RingZnElement(result, this->ring);
		return res;
	}

	
	void RingZnElement::operator-=(const RingZnElement& right)
		  throw(RingMismatchException)
	{
		if (this->ring != right.ring) {
			throw(RingMismatchException());
		}
		*(this->value) = *(this->value) - *(right.value);
		*(this->value) %= ring->getModulus();
	}
	
	
	RingZnElement RingZnElement::operator*(const RingZnElement& right) const
		  throw(RingMismatchException)
	{ 
		mpz_class n = (this->ring)->getModulus();
		mpz_class result = (this->getValue() * right.getValue()) % n;
		RingZnElement res  = RingZnElement(result, this->ring);
		return res;
	}
	
	
	void RingZnElement::operator*=(const RingZnElement& right)
		  throw(RingMismatchException)
	{
		if (this->ring != right.ring) {
			throw(RingMismatchException());
		}
		*(this->value) = *(this->value) * *(right.value);
		*(this->value) %= ring->getModulus();
	}

	
	RingZnElement RingZnElement::operator/(const RingZnElement& denominator) const 
		throw(NonInvertibleElementException<RingZnElement>,
		      RingMismatchException) 
	{
		if (this->ring != denominator.ring) {
			throw(RingMismatchException());
		}
		RingZnElement invDenom = denominator.inverse();
		RingZnElement res = (invDenom) * (*this);
		return res;
	}

	
	void RingZnElement::operator/=(const RingZnElement& denom)
		throw(NonInvertibleElementException<RingZnElement>,
		      RingMismatchException)
	{
		RingZnElement invDenom = denom.inverse();
		*(this->value) = *(this->value) + *(invDenom.value);
		*(this->value) %= (this->ring)->getModulus();
	}


		
	RingZnElement RingZnElement::inverse() const 
		throw(NonInvertibleElementException<RingZnElement>)
	{
		mpz_class n = this->ring->getModulus();
		mpz_class result = 0;
		//see the GMP documentation
		if ( !(mpz_invert(result.get_mpz_t(), 
				  (this->getValue()).get_mpz_t(), 
				  n.get_mpz_t() ) ) ) {
			//mpz_invert has returned zero, then the inverse does not exist !
			throw NonInvertibleElementException<RingZnElement >(*this);
		}
		RingZnElement res = RingZnElement(result, this->ring);
		return res;
	}
	
	
	RingZnElement RingZnElement::operator-() const
	{
		mpz_class newValue = -(*(this->value)) % this->ring->getModulus();
		return RingZnElement(newValue, this->ring);
	}
	
	
	RingZnElement& RingZnElement::operator=(const RingZnElement& right)
	{
		if (this != &right) {
			delete this->value;
			this->value = new mpz_class(*right.value);
		}
		return *this;
	}
	
	
	inline bool RingZnElement::isInvertible() const
	{
		mpz_class temp = 0;
		if ( !(mpz_invert(temp.get_mpz_t(), 
				  (this->getValue()).get_mpz_t(), 
				  (ring->getModulus()).get_mpz_t() ) ) ) {
			//mpz_invert has returned zero, then the inverse does not exist !
			return false;
		}
		else {
			return true;
		}
	}

	mpz_class RingZnElement::getPositiveValue() const
	{
		mpz_class ret = *(this->value);
		while (ret < 0) {
			ret += this->ring->getModulus();
		}
		return ret;
	}
	
	bool operator==(const RingZnElement& left, const RingZnElement& right)
		throw(RingMismatchException)
	{
		if (left.ring != right.ring) {
			throw(RingMismatchException());
		}
		return (left.getValue() == right.getValue());
	}

	
	std::ostream& operator<<(std::ostream& out, const RingZnElement& element)
	{
		out << element.getValue();
		return out;
	}



} //namespace libalgebra
