/***************************************************************************
 *   Copyright (C) 2007 by Andrea Moro   *
 *   andrea_moro@email.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>. *
 ***************************************************************************/

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#define VERSION "9999"

#include "ui_cryptoolinux.h"

class MdiChild;
class QMdiArea;
class QMdiSubWindow;
class QSignalMapper;
class QAssistantClient;

/**
 * @short Struct to save options related to plot and drawings.
 */
struct Tplot{
	char bar_width;
	char curve_width;
	bool zero;
	bool autocorr_relative;
};

/**
 * @short Struct to save options related to cryptoanalysis.
 */
struct Tanalysis{
	//	caesar
	bool caesar_derived_key;
	bool caesar_correlation;
	bool caesar_histograms;
	//	vigenere xor addition
	bool vxa_derived_key;
	bool vxa_derived_key_len;
	bool vxa_autocorrelation;
	bool vxa_base_analysis;
	//	brute-force
	int num_bytes_entropy;
	char cleartext_alph;
};

/**
 * @short Struct to save options related to text, like alphabet.
 */
struct Ttext{
	//	formatting
	bool char_unchanged;
	//	case
	bool keep_case;
	bool distinguish_case;
	//	alphabet
	QString alphabet;
	//	statistical file
	QString stat_file;
	//	playfair
	bool separate;
	QChar separator;
	//	hill
	QChar padChar;
	//	first character
	bool aEqual1;
};

/**
 * @short Struct to save options related the starting of CrypTooLinux.
 */
struct Tstart{
	bool sample_text;
};

/**
 * @short Struct to save some misc options.
 */
struct Tfurther{
	bool intro;
	QString alice_name;
};

/**
 * @short Struct to save all the options.
 */
struct Toptions{
	Tplot plot_opt;
	Tanalysis analysis_opt;
	Ttext text_opt;
	Tstart start_opt;
	Tfurther further_opt;
};

/**
 * @short Main user interface of CrypTooLinux.
 *
 * Main window of CrypTooLinux. Here are defined the Qt slots (like
 * open/save/new file, or connections to the cryptofunctions) where
 * the different actions (defined in ui_cryptoolinux.h) are plugged.
 */
class mainWindow :
	public QMainWindow,
	public Ui::CTLMainWindow
{
	Q_OBJECT
public:
	/**
	 * Constructor. Creates a new mainWindow instance.
	 */
	mainWindow(QWidget * parent);
	/**
	 * @returns The current MdiChild.
	 *
	 */
	MdiChild *currentMdiChild();
	
	/**
	 * Creates a new window for a new MdiChild.
	 *
	 * @param title the QString title of the new subwindow.
	 * @param content the content of the file.
	 */
	void newSubWindow(QString title, QString content);

private slots:
	/**
	 * Create a new window.
	 */
	void newFile();

	/**
	 * Open a file in a new window.
	 */
	void open();

	/**
	 * Save the content of a window.
	 */
	void save();

	/**
	 * Save the content of a window.
	 */
	void saveAs();

	/**
	 * Undo the last operation in the current window.
	 */
	void undo();

	/**
	 * Redo the last operation in the current window.
	 */
	void redo();

	/**
	 * Cut the selected text in the current window.
	 */
	void cut();

	/**
	 * Copy the selected text in the current window.
	 */
	void copy();

	/**
	 * Paste in the current window
	 */
	void paste();

	/**
	 * Select all the content in the current window.
	 */
	void selectAll();

	/**
	 * Update the Edit and File icon visibility.
	 */
	void toolbar();

	/**
	 * Update the statusbar visibility.
	 */
	void statusbar();

	/**
	 * Shows an about dialog.
	 */
	void about();

	/**
	 * Update the visibility of implemented algorithms in menu.
	 */
	void updateMenu();

	/**
	 * Start the assistant.
	 */
	void assistant();

	/**
	 * Create a new window.
	 *
	 * @return The new window.
	 */
	MdiChild* createMdiChild();

	/**
	 * Set active a window.
	 *
	 * @param window the window to set active.
	 */
	void setActiveSubWindow(QWidget *window);

	/**
	 * Save options in the user home's and exit
	 */
	void saveOptions();

	/* private slots cryptofunctions */

	/**
	 * Check if the content of the current window is not empty and
	 * launch Caesar/ROT13 window
	 */
	void crDe_caesarRot();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Vigenere window
	 */
	void crDe_vigenere();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Hill window
	 */
	void crDe_hill();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Atbash/substitution window
	 */
	void crDe_atbash();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Playfair window
	 */
	void crDe_playfair();

	/**
	 * Check if the content of the current window is not empty and
	 * launch ADFGVX window
	 */
	void crDe_ADFGVX();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Byte Add window
	 */
	void crDe_byteAdd();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Byte Xor window
	 */
	void crDe_byteXor();

	/**
	 * Check if the content of the current window is not empty and
	 * launch Vernam window
	 */
	void crDe_Vernam();

	/**
	 * Launches the factorisation window.
	 * 
	 */
	void crFactorisation_of_a_Number_2();

	/**
	 * Launches the prime number test window.
	 * 
	 */
	void cractionPrime_Number_Test();

	/******** Private Slots Cryptanalysis functions ************/
	void crCaesarAnalysis();

public slots:
	/**
	 * Open the window for Drawing Options
	 */
	void opt_draw();

	/**
	 * Open the window for Starting Options
	 */
	void opt_start();

	/**
	 * Open the window for Analysis Options
	 */
	void opt_analysis();

	/**
	 * Open the window for Further Options
	 */
	void opt_furt();

	/**
	 * Open the window for Text Options
	 */
	void opt_text();

	/**
	 * Open the window for Language Options
	 */
	void opt_lang();

private:
	/**
	 * Load options from the user home's
	 */
	bool loadOptions();

	/**
	 * Initialize assistent with the right locale help(if there exists)
	 */
	void initializeAssistant();

	/**
	 * Toolbar: icon for New file, Open file and Save file
	 */
	QToolBar *fileToolBar;

	/**
	 * Toolbar: icon for Cut, Copy and Paste
	 */
	QToolBar *editToolBar;

	/**
	 * Find a window that works on fileName
	 *
	 * @param &fileName the name of the file
	 * @return the window that work on fileName or 0
	 */
	QMdiSubWindow *findMdiChild(const QString &fileName);

	/**
	 * Makes the connections between signal of the ui and the slot functions.
	 */
	void createConnections();

	/**
	 * Create the fileToolBar and the editToolBar
	 */
	void createToolBars();

	/**
	 * the assistant client that show the help
	 */
	QAssistantClient *assistantClient;

	/**
	 * The path to the right locale help
	 */
	QString helpPath;

	/**
	 * the place where the window are.
	 */
	QMdiArea *workspace;

public:
	/**
	 * The struct that contain all the options
	 */
	Toptions options;

	/**
	 * This action is triggered when the options changed.
	 */
	QAction *opttext_changed;
};
#endif
